#include "padlock.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <memory.h>

#ifdef _WIN32_WCE
#include <Winbase.h>
typedef long  clock_t;
#endif

#ifdef Linux
#define clock GetMicroSec
clock_t GetMicroSec()
{
	struct timeval tv;
	gettimeofday(&tv,NULL);
	return(tv.tv_sec*1000L+tv.tv_usec/1000L);
}
#else
#define clock GetTickCount
#endif
#define CLOCKS_PER_SEC 1000 

#ifdef Linux
	#ifndef uint_64
		typedef unsigned long long uint_64;
	#endif
#else
	#include <windows.h>
	#ifndef uint_64
		typedef UINT64 uint_64;
	#endif
#endif

#ifdef _X64
#define ALIGN16(x) (unsigned char *)((((uint_64)x) + 15 )&0xfffffffffffffff0)
#else
#define ALIGN16(x) (unsigned char *)((((unsigned int)x) + 15 )&0xfffffff0)
#endif	



/***************** ECB *****************/

extern unsigned char ecb_aes128_key[16] ;

extern unsigned char ecb_aes192_key[24] ;

extern unsigned char ecb_aes256_key[32] ;

unsigned char ecb_aes_iv[16];

/***************** CBC *****************/
extern int cbc_aes128_key[4] ;

extern int cbc_aes192_key[6] ;

extern int cbc_aes256_key[8] ;

extern int cbc_aes_iv[4];
    
/***************** CFB *****************/

extern int cfb128_aes128_key[4] ;

extern int cfb128_aes192_key[6] ;

extern int cfb128_aes256_key[8];

extern int cfb128_aes_iv[4] ;
    
/*****************  OFB *****************/


extern int ofb_aes128_key[4];

extern int ofb_aes192_key[6];

extern int ofb_aes256_key[8];

extern int ofb_aes_iv[4];
    
/***************** CRT ******************/
extern int ctr_aes128_key[4] ;

extern int ctr_aes192_key[6] ;

extern int ctr_aes256_key[8];

extern int ctr_aes_iv[4];
    
/************************************************************************/

typedef struct  
{
	unsigned char ecb_aes128[16];
	unsigned char ecb_aes192[24];
	unsigned char ecb_aes256[32];
	unsigned char ecb_aes_iv[16];
} aes_ecb_struct;

typedef struct  
{
	int	 aes128_key[4] ;
	int	 aes192_key[6] ;
	int	 aes256_key[8];
	int	 aes_iv[4];		
} aes_other_struct;

typedef union
{
	aes_ecb_struct aes_ecb;
	aes_other_struct aes_other;
}aes_context;


int init_aes_context(ACE_AES_MODE aesMode,aes_context* aesContext)
{
	switch (aesMode)
	{
		case ACE_AES_CFB128:
			{
				memcpy((void*)aesContext->aes_other.aes128_key,(void*)cfb128_aes128_key,16);
				memcpy((void*)aesContext->aes_other.aes192_key,(void*)cfb128_aes192_key,24);
				memcpy((void*)aesContext->aes_other.aes256_key,(void*)cfb128_aes256_key,32);
				memcpy((void*)aesContext->aes_other.aes_iv,(void*)cfb128_aes_iv,16);
				return 1;
			}
		case ACE_AES_OFB128:
			{
				memcpy((void*)aesContext->aes_other.aes128_key,(void*)ofb_aes128_key,16);
				memcpy((void*)aesContext->aes_other.aes192_key,(void*)ofb_aes192_key,24);
				memcpy((void*)aesContext->aes_other.aes256_key,(void*)ofb_aes256_key,32);
				memcpy((void*)aesContext->aes_other.aes_iv,(void*)ofb_aes_iv,16);

				return 1;
			}
		case ACE_AES_ECB:
			{
				memcpy((void*)aesContext->aes_ecb.ecb_aes128,(void*)ecb_aes128_key,16);
				memcpy((void*)aesContext->aes_ecb.ecb_aes192,(void*)ecb_aes192_key,24);
				memcpy((void*)aesContext->aes_ecb.ecb_aes256,(void*)ecb_aes256_key,32);
				return 1;
			}
		case ACE_AES_CTR:
			{
				memcpy((void*)aesContext->aes_other.aes128_key,(void*)ctr_aes128_key,16);
				memcpy((void*)aesContext->aes_other.aes192_key,(void*)ctr_aes192_key,24);
				memcpy((void*)aesContext->aes_other.aes256_key,(void*)ctr_aes256_key,32);
				memcpy((void*)aesContext->aes_other.aes_iv,(void*)ctr_aes_iv,16);
				return 1;
			}
		case ACE_AES_CBC: 
			{
				memcpy((void*)aesContext->aes_other.aes128_key,(void*)cbc_aes128_key,16);
				memcpy((void*)aesContext->aes_other.aes192_key,(void*)cbc_aes192_key,24);
				memcpy((void*)aesContext->aes_other.aes256_key,(void*)cbc_aes256_key,32);
				memcpy((void*)aesContext->aes_other.aes_iv,(void*)cbc_aes_iv,16);
				return 1;
			}
		default:
			{
				return 0;
			}
	}
}


void test_normal_APIs_with_file(
	struct ace_aes_context* ecx,
	struct ace_aes_context* dcx,
	FILE *srcFile,
	FILE *encFile, 
	FILE *decFile,
	int dataLength,
	int cryptoMode
)
{
	unsigned char* dst = (unsigned char*)malloc(dataLength);
	unsigned char* src = (unsigned char*)malloc(dataLength);
	clock_t start, finish;
	clock_t total = 0;
	double timing = 0.;
	double rate = 0.;
	int fileLen = 0;
	int ispadded=0;
	int	retLength;
	int fileLen2 = 0;//for caculate rate
	
	if(NULL == dst || NULL == src)
	{
		free(src);
		free(dst);	
		return;
	}

	fseek(srcFile,0,SEEK_END);
	fileLen = ftell(srcFile);
	fileLen2 = fileLen;
	fseek(srcFile,0,SEEK_SET);
		
	switch(cryptoMode)
	{
		case 1: 
		{
			// encryption only
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					start = clock();
					padlock_aes_encrypt(ecx,src,dst,dataLength);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					start = clock();
					padlock_aes_encrypt(ecx,src,dst,retLength+16-retLength%16);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}
			}
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);	
				start = clock();
				padlock_aes_encrypt(ecx,src,dst,16);
				finish = clock() - start;
				total += finish;
				fwrite(dst,sizeof(char),16,encFile);
			}
			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
			break;
		}	
		case 2:
		{
			// decryption only
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;

				start = clock();
				padlock_aes_decrypt(dcx,src,dst,retLength);
				finish = clock() - start;
				total += finish;
				
				if (!fileLen)
				{
					if (dst[retLength-1]>16||dst[retLength-1]<1)
					{
						printf("Error! The decrypted file has not been encrypted correctly, and the result is not complete\n");
					}
					else
						fwrite(dst,sizeof(char),retLength-dst[retLength-1],decFile);
				}
				else
					fwrite(dst,sizeof(char),retLength,decFile);
				
			}		
			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
	
			break;
		}
		case 3:
		default:
		{
			// encryption then decryption. 
			while (fileLen)
			{
				// encrypt
				memset(dst, 0, dataLength);
				memset(src, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					start = clock();
					padlock_aes_encrypt(ecx,src,dst,dataLength);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					start = clock();
					padlock_aes_encrypt(ecx,src,dst,retLength+16-retLength%16);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}


				// decrypt
				memcpy(src,dst, dataLength);
				start = clock();
				if (retLength == dataLength)
				{
					padlock_aes_decrypt(dcx,src,dst,retLength);	
				}
				else
				{
					padlock_aes_decrypt(dcx,src,dst,retLength+16-retLength%16);
				}

				finish = clock() - start;
				total += finish;
				
				fwrite(dst,sizeof(char),retLength,decFile);
			}
			
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);	
				start = clock();
				padlock_aes_encrypt(ecx,src,dst,16);
				finish = clock() - start;
				total += finish;
				fwrite(dst,sizeof(char),16,encFile);
			}

			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 * 2.0 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
			break;
		}		
	}
	free(src);
	free(dst);
}


void test_aligned_APIs_with_file(
	char* pKey, 
	char* iv,
	ACE_AES_MODE aesMode,
	KEY_LENGTH keyLen,	
	FILE *srcFile,
	FILE *encFile, 
	FILE *decFile,
	int dataLength,
	int cryptoMode
)
{
	unsigned char* dst=(unsigned char*)malloc(dataLength);
	unsigned char* src=(unsigned char*)malloc(dataLength);

	unsigned char* eiv = (unsigned char*)malloc(16); 
	unsigned char* div = (unsigned char*)malloc(16); 

	struct aligned_memory_context* buf_aligned_mctx = NULL;

	clock_t start, finish;
	clock_t total = 0;
	double timing = 0.;
	double rate = 0.;
	int fileLen = 0;
	int ispadded = 0;
	int	retLength;
	int fileLen2 = 0;//for caculate rate

	if(NULL == dst || NULL == src || NULL == eiv || NULL ==div )
	{
		free(src);
		free(dst);	
		free(eiv);
		free(div);
		return;
	}

	fseek(srcFile,0,SEEK_END);
	fileLen = ftell(srcFile);
	fileLen2 = fileLen;
	fseek(srcFile,0,SEEK_SET);

	buf_aligned_mctx = padlock_aligned_malloc(dataLength);
	
	if(NULL == buf_aligned_mctx)
	{
		free(eiv);
		free(div);
		free(src);
		free(dst);
		return;
	}

	memcpy(eiv, iv, 16);
	memcpy(div, iv, 16);

	switch(cryptoMode)
	{
		case 1: 
		{
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					padlock_aligned_memcpy_to(buf_aligned_mctx, src, dataLength);
					start = clock();
					padlock_aes_aligned_encrypt(pKey, keyLen, aesMode, buf_aligned_mctx, eiv);
					finish = clock() - start;
					total += finish;
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, dataLength);
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					padlock_aligned_memcpy_to(buf_aligned_mctx, src, retLength+16-retLength%16);
					start = clock();
					padlock_aes_aligned_encrypt(pKey, keyLen, aesMode, buf_aligned_mctx, eiv);
					finish = clock() - start;
					total += finish;
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, retLength+16-retLength%16);
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}
			}
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);
				padlock_aligned_memcpy_to(buf_aligned_mctx, src, 16);
				start = clock();
				padlock_aes_aligned_encrypt(pKey, keyLen, aesMode, buf_aligned_mctx, eiv);
				finish = clock() - start;
				total += finish;
				padlock_aligned_memcpy_from(buf_aligned_mctx, dst, 16);
				fwrite(dst,sizeof(char),16,encFile);
			}

			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
		
			break;
		}	
		case 2:
		{
			// decryption only
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;

				padlock_aligned_memcpy_to(buf_aligned_mctx, src, retLength);
				start = clock();
				padlock_aes_aligned_decrypt(pKey, keyLen, aesMode, buf_aligned_mctx, div);
				finish = clock() - start;
				total += finish;
				padlock_aligned_memcpy_from(buf_aligned_mctx, dst, retLength);	

				if (!fileLen)
				{
					if (dst[retLength-1]>16||dst[retLength-1]<1)
					{
						printf("Error! The decrypted file has not been encrypted correctly, and the result is not complete\n");
					}
					else
						fwrite(dst,sizeof(char),retLength-dst[retLength-1],decFile);
				}
				else
					fwrite(dst,sizeof(char),retLength,decFile);
				
			}		

			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
		
			break;
		}
		case 3:
		default:
		{
			// encryption then decryption. 
			while (fileLen)
			{
				// encrypt
				memset(dst, 0, dataLength);
				memset(src, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					padlock_aligned_memcpy_to(buf_aligned_mctx, src, dataLength);
					start = clock();
					padlock_aes_aligned_encrypt(pKey, keyLen, aesMode, buf_aligned_mctx, eiv);
					finish = clock() - start;
					total += finish;
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, dataLength);
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					padlock_aligned_memcpy_to(buf_aligned_mctx, src, retLength+16-retLength%16);
					start = clock();
					padlock_aes_aligned_encrypt(pKey, keyLen, aesMode, buf_aligned_mctx, eiv);
					finish = clock() - start;
					total += finish;
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, retLength+16-retLength%16);
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}


				// decrypt
				//memcpy(src,dst, dataLength);
				start = clock();
				if (retLength == dataLength)
				{
					padlock_aligned_memcpy_to(buf_aligned_mctx, dst, retLength);
					padlock_aes_aligned_decrypt(pKey, keyLen, aesMode, buf_aligned_mctx, div);
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, retLength);
				}
				else
				{
					padlock_aligned_memcpy_to(buf_aligned_mctx, dst, retLength+16-retLength%16);
					padlock_aes_aligned_decrypt(pKey, keyLen, aesMode, buf_aligned_mctx, div);
					padlock_aligned_memcpy_from(buf_aligned_mctx, dst, retLength+16-retLength%16);
				}

				finish = clock() - start;
				total += finish;
				
				fwrite(dst,sizeof(char),retLength,decFile);
			}
			
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);	
				
				padlock_aligned_memcpy_to(buf_aligned_mctx, src, 16);
				start = clock();
				padlock_aes_aligned_decrypt(pKey, keyLen, aesMode, buf_aligned_mctx, div);
				start = clock();
				padlock_aligned_memcpy_from(buf_aligned_mctx, dst, 16);
				finish = clock() - start;
				total += finish;
				fwrite(dst,sizeof(char),16,encFile);
			}
			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 * 2.0 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
		
			break;
		}		
	}
	padlock_aligned_mfree( buf_aligned_mctx);
	free(eiv);
	free(div);
	free(src);
	free(dst);
	return;
}


void test_fast_APIs_with_file(
	char* pKey, 
	char* iv,
	ACE_AES_MODE aesMode,
	KEY_LENGTH keyLen,	
	FILE *srcFile,
	FILE *encFile, 
	FILE *decFile,
	int dataLength,
	int cryptoMode
)
{
	unsigned char* dstBuf = (unsigned char*)malloc(dataLength + 15);
	unsigned char* srcBuf = (unsigned char*)malloc(dataLength + 15);
	unsigned char* eivBuf = (unsigned char*)malloc(32); 
	unsigned char* divBuf = (unsigned char*)malloc(32); 
	unsigned char *dst = NULL;
	unsigned char *src = NULL;
	unsigned char *eiv = NULL;
	unsigned char *div = NULL;

	clock_t start, finish;
	clock_t total = 0;
	double timing = 0.;
	double rate = 0.;
	int fileLen = 0;
	int ispadded =0;
	int	retLength;
	int fileLen2 = 0;//for caculate rate

	if(NULL == dstBuf || NULL == srcBuf || NULL == eivBuf || NULL == divBuf)
	{
		free(dstBuf);
		free(srcBuf);
		free(eivBuf);
		free(divBuf);
		return;
	}

	dst = ALIGN16(dstBuf);
	src = ALIGN16(srcBuf);
	eiv = ALIGN16(eivBuf);
	div = ALIGN16(divBuf);

	fseek(srcFile,0,SEEK_END);
	fileLen = ftell(srcFile);
	fileLen2 = fileLen;
	fseek(srcFile,0,SEEK_SET);

	memcpy(eiv, iv, 16);
	memcpy(div, iv, 16);
	
	switch(cryptoMode)
	{

		case 1: 
		{
			// encryption only
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					start = clock();
					padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,dataLength,eiv);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					start = clock();
					padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,retLength+16-retLength%16,eiv);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}
			}
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);	
				start = clock();
				padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,16,eiv);
				finish = clock() - start;
				total += finish;
				fwrite(dst,sizeof(char),16,encFile);
			}
			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
			break;
		}	
		case 2:
		{
			// decryption only
			while (fileLen)
			{
				memset(src, 0, dataLength);
				memset(dst, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;

				start = clock();
				padlock_aes_fast_decrypt(pKey,keyLen,aesMode,src,dst,retLength,div);
				finish = clock() - start;
				total += finish;
				
				if (!fileLen)
				{
					if (dst[retLength-1]>16||dst[retLength-1]<1)
					{
						printf("Error! The decrypted file has not been encrypted correctly, and the result is not complete\n");
					}
					else
						fwrite(dst,sizeof(char),retLength-dst[retLength-1],decFile);
				}
				else
					fwrite(dst,sizeof(char),retLength,decFile);
				
			}		
			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
	
			break;
		}
		case 3:
		default:
		{
			// encryption then decryption. 
			while (fileLen)
			{
				// encrypt
				memset(dst, 0, dataLength);
				memset(src, 0, dataLength);
				retLength = fread(src,sizeof(char),dataLength,srcFile);
				fileLen -= retLength;
				if (retLength == dataLength)
				{
					start = clock();
					padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,retLength,eiv);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),dataLength,encFile);
				}
				else
				{
					memset(src+retLength,16-retLength%16,16-retLength%16);
					ispadded = 1;
					start = clock();
					padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,retLength+16-retLength%16,eiv);
					finish = clock() - start;
					total += finish;
					fwrite(dst,sizeof(char),retLength+16-retLength%16,encFile);
				}


				// decrypt
				memcpy(src,dst, dataLength);
				start = clock();
				if (retLength == dataLength)
				{
					padlock_aes_fast_decrypt(pKey,keyLen,aesMode,src,dst,retLength,div);
				}
				else
				{
					padlock_aes_fast_decrypt(pKey,keyLen,aesMode,src,dst,retLength+16-retLength%16,div);
				}

				finish = clock() - start;
				total += finish;
				
				fwrite(dst,sizeof(char),retLength,decFile);
			}
			
			if (!ispadded)
			{
				memset(src,0,dataLength);
				memset(dst,0,dataLength);
				memset(src,16,16);	
				start = clock();
				padlock_aes_fast_encrypt(pKey,keyLen,aesMode,src,dst,16,eiv);
				finish = clock() - start;
				total += finish;
				fwrite(dst,sizeof(char),16,encFile);
			}

			timing = (double)total / CLOCKS_PER_SEC;		
			rate = (double) (fileLen2) * (double) 8 * 2.0 / (1000000000. * timing);
			printf("time: \t%10.6f seconds     %6.3fG bits/sec\n", timing, rate);
			break;
		}	

	}
	free(dstBuf);
	free(srcBuf);
	free(eivBuf);
	free(divBuf);
	return;
}
void aes_file_do_test(FILE* srcFile,FILE* encFile, FILE *decFile, int dataLength, ACE_AES_MODE aesMode,KEY_LENGTH keyLen,int cryptoMode,int testMode)
{
	struct ace_aes_context* ecx = NULL;
	struct ace_aes_context* dcx = NULL;
	aes_context aesContext;
	
	unsigned char* pKey=NULL;
	unsigned char* pKey0=(unsigned char*)malloc(128+15);
	
	unsigned char *eiv=NULL;
	unsigned char *div=NULL;
	unsigned char *eiv0=(unsigned char*)malloc(16+15);
	unsigned char *div0=(unsigned char*)malloc(16+15);

	int fileLen = 0;

	if(NULL == eiv0 || NULL == div0 || NULL == pKey0)
	{
		free(eiv0);
		free(div0);
		free(pKey0);
		return;
	}

	ecx = padlock_aes_begin();
	dcx = padlock_aes_begin();
	if(NULL == ecx || NULL == dcx)
	{
		padlock_aes_close(ecx);
		padlock_aes_close(dcx);
		free(eiv0);
		free(div0);
		free(pKey0);
		return;
	}

	fseek(srcFile,0,SEEK_END);
	fileLen=ftell(srcFile);
	if (fileLen==-1)
	{
		padlock_aes_close(ecx);
		padlock_aes_close(dcx);
		free(eiv0);
		free(div0);
		free(pKey0);
		printf("Source file is empty!\n");
		return;
	}
	
	eiv=ALIGN16(eiv0);
	div=ALIGN16(div0);
	pKey=ALIGN16(pKey0);

	memset(pKey0,0,128);

	init_aes_context(aesMode,&aesContext);
	switch (aesMode)
	{
		 case ACE_AES_ECB:
		{
			switch (keyLen)
			{
				case KEY_128BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_ecb.ecb_aes128,16);
					break;
				}
				case KEY_192BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_ecb.ecb_aes192,24);
					break;
				}
				case KEY_256BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_ecb.ecb_aes256,32);
					break;
				}
				default:
				{
					printf("error in key length.\n");
					padlock_aes_close(ecx);
					padlock_aes_close(dcx);
					free(pKey0);
					free(eiv0);
					free(div0);
					return;
				}

			}
			break;
		}
		default:
		{
			memcpy(eiv,(unsigned char*)aesContext.aes_other.aes_iv,16);
			memcpy(div,(unsigned char*)aesContext.aes_other.aes_iv,16);
			switch (keyLen)
			{
				case KEY_128BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_other.aes128_key,16);
					break;
				}
				case KEY_192BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_other.aes192_key,24);
					break;
				}
				case KEY_256BITS:
				{
					memcpy(pKey,(unsigned char*)aesContext.aes_other.aes256_key,32);
					break;
				}
				default:
				{
					printf("error in key length.\n");
					break;
				}
			}
		}
	}
	padlock_aes_setkey( ecx, pKey, keyLen);
	padlock_aes_setkey( dcx, pKey, keyLen);
	padlock_aes_setmodeiv ( ecx, aesMode,eiv);
	padlock_aes_setmodeiv ( dcx, aesMode,div);


	switch (testMode)
	{
		case 1://normal
		{
			test_normal_APIs_with_file(ecx, dcx, srcFile, encFile, decFile, dataLength, cryptoMode);
			break;
		}
		case 2://aligned 
		{
			test_aligned_APIs_with_file(pKey, eiv, aesMode, keyLen, srcFile, encFile, decFile, dataLength, cryptoMode);
			break;
		}
		case 3: //fast
		{
			test_fast_APIs_with_file(pKey, eiv, aesMode, keyLen, srcFile, encFile, decFile, dataLength, cryptoMode);
			break;
		}
		default:
			break;
	}


	padlock_aes_close (ecx);
	padlock_aes_close (dcx);
	free (eiv0);
	free (div0);
	free (pKey0);
}

int aes_performance_fast(void* src,void* dst,KEY_LENGTH keyLen,int dataLen)
{
	char* str[5]=
	{"ACE_AES_ECB","ACE_AES_CBC",
	 "ACE_AES_CFB128","ACE_AES_OFB128",
	 "ACE_AES_CTR"};

	unsigned char* pKey=NULL;
	unsigned char* pKey0=(unsigned char*)malloc(128+15);
	
	unsigned char *iv=NULL;
	unsigned char *iv0=(unsigned char*)malloc(16+15);

	clock_t start, finish;
	double interval = 0;
	double rate = 0;

	int ret=0;
	int i=0;
	int j=0;

	aes_context aesContext;

	if(NULL == pKey0 || NULL == iv0)
	{
		free(pKey0);
		free(iv0);	
		return 0;
	}

	pKey=ALIGN16(pKey0);
	iv=ALIGN16(iv0);
		
	for (i=0;i<5;++i)
	{
		printf("********************************************************************\n");
		printf("%s test 10000 times:\n",str[i]);
		init_aes_context(i,&aesContext);
		switch (keyLen)
		{
			case KEY_128BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes128_key,16);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_192BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes192_key,24);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_256BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes256_key,32);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			default:
			{
				printf("error in key length.\n");
				free (iv0);
				free (pKey0);
				return 0;
			}
		}
		start=clock();
		for (j=0;j<10000;++j)
		{				
			padlock_aes_fast_encrypt(pKey,keyLen,i,src,dst,dataLen,iv);
		}
		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Encrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);

		start=clock();
		for (j=0;j<10000;++j)
		{				
			padlock_aes_fast_decrypt(pKey,keyLen,i,src,dst,dataLen,iv);
		}

		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Decrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);


	}
	free (iv0);
	free (pKey0);
	return 1;
}


int aes_performance_normal(void* src,void* dst,KEY_LENGTH keyLen,int dataLen)
{
	char* str[5]=
	{"ACE_AES_ECB","ACE_AES_CBC",
	 "ACE_AES_CFB128","ACE_AES_OFB128",
	 "ACE_AES_CTR"};

	unsigned char* pKey=NULL;
	unsigned char* pKey0=(unsigned char*)malloc(128+15);
	
	unsigned char *iv=NULL;
	unsigned char *iv0=(unsigned char*)malloc(16+15);

	clock_t start, finish;
	double interval = 0;
	double rate = 0;

	int ret=0;
	int mode =0;
	int j=0;

	aes_context aesContext;
	struct ace_aes_context *ctx = padlock_aes_begin();
	
	if(NULL == pKey0 || NULL == iv0 || NULL == ctx)
	{
		padlock_aes_close(ctx);
		free(pKey0);
		free(iv0);	
		return 0;
	}

	pKey=ALIGN16(pKey0);
	iv=ALIGN16(iv0);
		
	for (mode = 0 ; mode < 5; ++mode)
	{
		printf("********************************************************************\n");
		printf("%s test 10000 times:\n",str[mode]);
		init_aes_context(mode,&aesContext);
		switch (keyLen)
		{
			case KEY_128BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes128_key,16);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_192BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes192_key,24);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_256BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes256_key,32);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			default:
			{
				printf("error in key length.\n");
				padlock_aes_close(ctx);
				free (iv0);
				free (pKey0);
				return 0;
			}
		}
		padlock_aes_setkey(ctx, pKey, keyLen);
		padlock_aes_setmodeiv(ctx, mode, iv);

		start=clock();
		for (j=0;j<10000;++j)
		{	
			padlock_aes_encrypt(ctx, src, dst, dataLen);
		}
		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Encrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);

		start=clock();
		for (j=0;j<10000;++j)
		{				
			padlock_aes_decrypt(ctx, src, dst, dataLen);
		}

		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Decrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);

		
	}
	free (iv0);
	free (pKey0);
	padlock_aes_close(ctx);
	return 1;
}


int aes_performance_aligned(void* src,void* dst,KEY_LENGTH keyLen,int dataLen)
{
	char* str[5]=
	{"ACE_AES_ECB","ACE_AES_CBC",
	 "ACE_AES_CFB128","ACE_AES_OFB128",
	 "ACE_AES_CTR"};

	unsigned char* pKey=NULL;
	unsigned char* pKey0=(unsigned char*)malloc(128+15);
	
	unsigned char *iv=NULL;
	unsigned char *iv0=(unsigned char*)malloc(16+15);

	clock_t start, finish;
	double interval = 0;
	double rate = 0;

	int ret=0;
	int mode=0;
	int j=0;

	aes_context aesContext;
	struct aligned_memory_context *memCtx = padlock_aligned_malloc(dataLen);

	if(NULL == pKey0 || NULL == iv0 || NULL == memCtx)
	{
		free(pKey0);
		free(iv0);	
		padlock_aligned_mfree(memCtx);
		return 0;
	}
	pKey=ALIGN16(pKey0);
	iv=ALIGN16(iv0);
	padlock_aligned_memcpy_to(memCtx, src, dataLen);
		
	for (mode = 0 ; mode < 5; ++mode)
	{
		printf("********************************************************************\n");
		printf("%s test 10000 times:\n",str[mode]);
		init_aes_context(mode,&aesContext);
		switch (keyLen)
		{
			case KEY_128BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes128_key,16);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_192BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes192_key,24);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			case KEY_256BITS:
			{
				memcpy(pKey,(unsigned char*)aesContext.aes_other.aes256_key,32);
				memcpy(iv,(unsigned char*)aesContext.aes_other.aes_iv,16);
				break;
			}
			default:
			{
				printf("error in key length.\n");
				free (iv0);
				free (pKey0);
				padlock_aligned_mfree(memCtx);
				return 0;
			}
		}

		start=clock();
		for (j=0;j<10000;++j)
		{	
			padlock_aes_aligned_encrypt(pKey, keyLen, mode, memCtx, iv); 
		}
		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Encrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);

		start=clock();
		for (j=0;j<10000;++j)
		{				
			padlock_aes_aligned_decrypt(pKey, keyLen, mode, memCtx, iv); 
		}

		finish = clock();
		interval = ((double)finish - (double)start) / CLOCKS_PER_SEC;
		rate = (double)(dataLen) * 8.0 * 10000.0 / (1024. * 1024. * 1024. * interval);
		printf("Decrypt time: %6.3f seconds    %6.3fG bits/seconds\n",interval, rate);

		
	}
	free (iv0);
	free (pKey0);
	padlock_aligned_mfree(memCtx);
	return 1;
}


int aes_performance_test()
{
	int keysize=0;
	int aesmode=0;
	int testMode=0;
	int dataLen=1;
	KEY_LENGTH keyLen;
	int type = 0; 

	unsigned char* src=NULL;
	unsigned char* dst=NULL;
	unsigned char* src1=NULL;
	unsigned char* dst1=NULL;

	printf("********************************************************************\n");
	printf("input the test size which must be divided by 16, such as 1024,2048..\n");
	printf("input: ");
	scanf("%d",&dataLen);
	if ((dataLen % 16) != 0)
	{
		printf("error for that it can not be divided by 16.\n");
		return 0;
	}
	
	printf("********************************************************************\n");
	printf("input the keysize:\n");
	printf("1 for KEY_128BIT,\t2 for KEY_192BIT,\t3 for KEY_256BIT.\n");
	printf("input: ");
	scanf("%d",&keysize);
	
	printf("********************************************************************\n");
	printf("input the API types:\n");
	printf("1 for normal APIs, \t2 for aligned APIs ,\t3 for fast APIs.\n");
	printf("input: ");
	scanf("%d",&type);


	switch(keysize)
	{
		case 1:
			printf("KEY_128BIT is selected.\n");
			keyLen=KEY_128BITS;
			break;
		case 2:
			printf("KEY_192BIT is selected.\n");
			keyLen=KEY_192BITS;
			break;
		case 3:
			printf("KEY_256BIT is selected.\n");
			keyLen=KEY_256BITS;
			break;
		default:
			printf("error key size!\n");
			return 0;
	}


	src=(unsigned char*)malloc(dataLen+15);
	dst=(unsigned char*)malloc(dataLen+15);
	
	if(NULL == src || NULL == dst)
	{
		free(src);
		free(dst);
		printf("memory allocate error!\n");
		return 0;
	}

	src1=ALIGN16(src);
	dst1=ALIGN16(dst);

	padlock_rng_rand(src1, dataLen);
	switch (type)
	{
	case 1: 
		aes_performance_normal(src1, dst1, keyLen, dataLen);
		break;
	case 2:
		aes_performance_aligned(src1, dst1, keyLen, dataLen);
		break;
	case 3:
	default:
		aes_performance_fast(src1,dst1,keyLen,dataLen);
		break;

	}
	
	free(src);
	free(dst);
	printf("\n\n");
	return 1;	
}

int aes_file_test()
{
	int keysize=0;
	int aesmode=0;
	int testmode=0;
	int cryptoMode=1;
	KEY_LENGTH keyLen;
	ACE_AES_MODE aesMode;

	FILE*  srcFile=NULL;
	FILE*  encFile=NULL;
	FILE*  decFile=NULL;

	
	char filename[256];
	char encfile[260];
	char decfile[260];

	int fileLen=-1;
	int readed=-1;
	int writed=-1;
	int ret=0;
	int dataLen = 0;

	
	printf("********************************************************************\n");	
	printf("input the file name:\n");
	printf("input: ");
	scanf("%s",filename);
	srcFile=fopen(filename,"rb");
	if (!srcFile)
	{
		printf("error in open src file %s\n",filename);
		return 0;
	}

	fseek(srcFile,0,SEEK_END);
	fileLen=ftell(srcFile);
	if (fileLen==-1)
	{
		fclose(srcFile);
		printf("error when deal with empty file\n");
		return 0;
	}
	fseek(srcFile,0,SEEK_SET);

	printf("********************************************************************\n");
	printf("input the mode:\n");
	printf("1 for ACE_AES_ECB,\t2 for ACE_AES_CBC,\t3 for ACE_AES_CFB128,\n4 for ACE_AES_OFB128,\t5 for ACE_AES_CTR.\n");
	printf("input: ");
	scanf("%d",&aesmode);
	
	printf("********************************************************************\n");
	printf("input test mode:\n");
	printf("1 for normal test,\t2 for aligned test,\t3 for fast test.\n");
	printf("input: ");
	scanf("%d",&testmode);

	printf("********************************************************************\n");
	printf("input the keysize:\n");
	printf("1 for KEY_128BIT,\t2 for KEY_192BIT,\t3 for KEY_256BIT.\n");
	printf("input: ");
	scanf("%d",&keysize);

	sprintf(encfile,"%s.enc",filename);
	sprintf(decfile,"%s.dec",filename);

	printf("********************************************************************\n");
	printf("input the crypt method:\n");
	printf("1 for encrypt\t2 for decrypt.\t3 for both encrypt and decrypt\n");
	printf("input: ");
	scanf("%d",&cryptoMode);
	printf("********************************************************************\n");
	switch(testmode)
	{
		case 1:
			printf("normal test is selected.\n");
			break;
		case 2:
			printf("aligned test is selected.\n");
			break;
		case 3:
			printf("fast test is selected.\n");
			break;
		default:
			printf("error test mode!\n");
			fclose(srcFile);
			return 0;
	}

	switch(keysize)
	{
		case 1:
			printf("KEY_128BIT is selected.\n");
			keyLen=KEY_128BITS;
			break;
		case 2:
			printf("KEY_192BIT is selected.\n");
			keyLen=KEY_192BITS;
			break;
		case 3:
			printf("KEY_256BIT is selected.\n");
			keyLen=KEY_256BITS;
			break;
		default:
			printf("error key size!\n");
			fclose(srcFile);
			return 0;
	}

	switch(aesmode)
	{
		case 1:
			{
				printf("ACE_AES_ECB is selected.\n");
				aesMode=ACE_AES_ECB;
				break;
			}
		case 2:
			{
				printf("ACE_AES_CBC is selected.\n");
				aesMode=ACE_AES_CBC;
				break;
			}
		case 3:
			{
				printf("ACE_AES_CFB128 is selected.\n");
				aesMode=ACE_AES_CFB128;
				break;
			}
		case 4:
			{
				printf("ACE_AES_OFB128 is selected.\n");
				aesMode=ACE_AES_OFB128;
				break;
			}
		case 5:
			{
				printf("ACE_AES_CTR is selected.\n");
				aesMode=ACE_AES_CTR;
				break;
			}
		default:
			{
				printf("error mode!\n");
				fclose(srcFile);
				return 0;
			}
	}
	
	printf("********************************************************************\n");
	printf("input the test size which must be divided by 16, such as 1024,2048..\n");
	printf("input: ");
	scanf("%d",&dataLen);
	if ((dataLen % 16) != 0)
	{
		printf("error for that it can not be divided by 16.\n");
		return 0;
	}

	if (cryptoMode ==1)//encrypt
	{
		encFile=fopen(encfile,"w+b");
		if (!encFile)
		{
			printf("error in open encrypt file\n");
			fclose(srcFile);
			return 0;
		}
		fseek(srcFile,0,SEEK_SET);
		aes_file_do_test(srcFile, encFile, decFile, dataLen, aesMode,keyLen,1,testmode);
	}
	else if (cryptoMode == 2)
	{
		decFile=fopen(decfile,"w+b");
		if (!decFile)
		{
			printf("error in open decrypt file\n");
			fclose(srcFile);
			return 0;
		}
		fseek(srcFile,0,SEEK_SET);
		aes_file_do_test(srcFile, encFile, decFile, dataLen, aesMode,keyLen,2,testmode);
	}
	else
	{
		encFile=fopen(encfile,"w+b");
		if (!encFile)
		{
			printf("error in open encrypt file\n");
			fclose(srcFile);
			return 0;
		}
		decFile=fopen(decfile,"w+b");
		if (!decFile)
		{
			printf("error in open decrypt file\n");
			fclose(srcFile);
			fclose(encFile);
			return 0;
		}
		fseek(srcFile,0,SEEK_SET);
		aes_file_do_test(srcFile, encFile, decFile, dataLen, aesMode,keyLen,3,testmode);
	}
	
	if (encFile)
	{
		fclose(encFile);
	}
	if (decFile)
	{
		fclose(decFile);
	}
	fclose(srcFile);
	printf("\tFinished the test for file <%s>. \n", filename);
	printf("\tPlease check the output file: <%s> \n", (encFile) ? encfile : decfile);
	printf("\n\n");
	return 1;
}


