#ifndef MONTMUL_H
#define MONTMUL_H

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_BITS 8192
#define MAX_DWORDS 256

typedef unsigned short uInt_16;
typedef unsigned int uInt_32;

#ifdef Linux
typedef unsigned long long uInt_64;
#else
#include <windows.h>
typedef UINT64 uInt_64;
#endif

#define MONT_MASK 0x3000

typedef enum
{
	A_PTR=0,
	B_PTR,
	MOD_PTR,
	DST_PTR,
	PRM_PTR,
	TMP_PTR,
	RM_PTR
}PTR_NAME;

typedef struct
{
	uInt_32	 mzeroprime;
	uInt_32  length;
	uInt_32* ptr[8];
	uInt_32* buffer;
} montmul_context;


#ifdef _X64
#define ALIGN16(x) (unsigned char *)((((uInt_64)x) + 15 )&0xfffffffffffffff0)
#else
#define ALIGN16(x) (unsigned char *)((((unsigned int)x) + 15 )&0xfffffff0)
#endif
 
void padlock_mm(uInt_32* A, uInt_32* B, uInt_32* dst, uInt_32* mod, uInt_32 len);

void padlock_montmul_mm( uInt_32* A, uInt_32* B, uInt_32* dst, montmul_context* mmctx);

void padlock_montmul_set_mod_len( montmul_context* mmctx, uInt_32* mod, uInt_32 len);

void padlock_montmul_close( montmul_context* mmctx);

montmul_context* padlock_montmul_begin();

#ifdef __cplusplus
}
#endif

#endif