/*
 * Copyright 1998-2005 VIA Technologies, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * VIA, AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "padlock.h"
#include "via_ace.h"
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

extern int asm_rng_available();
extern unsigned int asm_rng(unsigned char *rand);

////////////////////////////////////////////////////////////////
// VIA Padlock SDK RNG API
int padlock_rng_available()
{
	return asm_rng_available();
}

RNG_RESULT padlock_rng(unsigned char *rand, int rand_len)
{
	unsigned int tmp_len = 0;
	while(rand_len>0)
	{
		tmp_len = asm_rng(rand);
		rand_len -= tmp_len;
		rand +=tmp_len;
	}
	
	return RNG_SUCCEEDED;
}

RNG_RESULT 
padlock_rng_rand(unsigned char *rand,int rand_len)
{

	int processing_len = 0;
	int processed_len = 0;
	unsigned char *temp_buffer = 0;
    unsigned char *buffer = 0;
	RNG_RESULT ret= RNG_SUCCEEDED;
	int len = 0;
	
	if(NULL == rand)
	{
		printf("Random number storage address null pointer error!\n");
		return RNG_FAILED;
	}

	if (0 == rand_len)
	{
		return RNG_SUCCEEDED;
	}
/*
	// first 16 bytes because RNG hardware may change the value after allocated memory
	// boundary second 16 bytes because RNG hardware need the destination memory address
	// is 16 bytes aligned
	temp_buffer = (unsigned char *)malloc(MAX_CIPHER_BUFFER_SIZE+16+16);
	if(temp_buffer == NULL)
	{
		printf("Fail to malloc buffer for RNG!\n");
		return RNG_FAILED;
	}

	buffer = ALIGN16(temp_buffer);

	processing_len = rand_len > MAX_CIPHER_BUFFER_SIZE ? MAX_CIPHER_BUFFER_SIZE : rand_len;
		
	do{
			
		ret = padlock_rng(buffer,processing_len);

		if(ret != AES_SUCCEEDED)
		{
			break;
		}
		
		memcpy(rand + processed_len, buffer, processing_len);
			
		processed_len = processed_len + processing_len;
			
		if((rand_len - processed_len) < MAX_CIPHER_BUFFER_SIZE)
		{
			processing_len = rand_len - processed_len;
		}
		
	}while(processed_len < rand_len);

	free( temp_buffer);
*/
	//do not malloc buffer, but handle block by block.
	//modified by meshellcui.20081104.

	temp_buffer = (unsigned char *)malloc(64);
	buffer = ALIGN16(temp_buffer);
	memset(temp_buffer,0,64);

	//0. if rand_len <= 32 , copy .
	if(rand_len<33)
	{
		ret = padlock_rng(buffer,rand_len);

		if(ret != AES_SUCCEEDED)
		{
			free( temp_buffer);
			return ret;
		}

		memcpy(rand, buffer, rand_len);
		free( temp_buffer);
		return ret;
	}

	//1. if rand is 16byte aligned
	len = (long)rand%16;

	if(len)
	{
		ret = padlock_rng(buffer,16-len);

		if(ret != AES_SUCCEEDED)
		{
			free( temp_buffer);
			return ret;
		}
		memcpy(rand, buffer, 16-len);
	}

	//2. handle middle part
	if(!len)len=16;
	ret = padlock_rng(rand-len+16,rand_len-(16-len)-16);

	if(ret != AES_SUCCEEDED)
	{
		free( temp_buffer);
		return ret;
	}

	//3. handle last part
	memset(temp_buffer,0,64);
	ret = padlock_rng(buffer,16);
	if(ret != AES_SUCCEEDED)
	{
		free( temp_buffer);
		return ret;
	}
	memcpy(rand+rand_len-16, buffer, 16);
	free( temp_buffer);
	//modified end.
	return ret;
}



